import paho.mqtt.client as mqtt
from vmiiputil import getIpV4
import threading
import json
import time

class MqttChannel:
    def __init__(self, host, port, username, password) -> None:
        self._host = host
        self._port = port
        self._username = username
        self._password = password
        self._onConnectCB = None
        self._onDisConnectCB = None
        self._onMessageCB = None
        self._client = None

    def _onConnect(self, client, userdata, flags, rc):
        if self._onConnectCB is not None:
            self._onConnectCB(client, userdata, flags, rc)

    def _onDisconnect(self, client, userdata, rc):
        if self._onDisConnectCB is not None:
            self._onDisConnectCB(client, userdata, rc)
    @property
    def on_connect(self):
        return self._onConnectCB

    @on_connect.setter
    def on_connect(self, cb):
        self._onConnectCB = cb
    
    @property
    def on_disconnect(self):
        return self._onDisConnectCB

    @on_disconnect.setter
    def on_disconnect(self, cb):
        self._onDisConnectCB = cb

    @property
    def on_message(self):
        return self._onMessageCB
    
    @on_message.setter
    def on_message(self, cb):
        self._onMessageCB = cb

    def _onMessage(self, client, userdata, message):
        if self._onMessageCB is not None:
            self._onMessageCB(client, userdata, message)

    def Run(self):
        while True:
            self._client = mqtt.Client()
            self._client.on_connect = self._onConnect
            self._client.on_disconnect = self._onDisconnect
            self._client.on_message = self._onMessage
            self._client.username_pw_set(username=self._username, password=self._password)
            try:
                self._client.connect(host=self._host, port=self._port, keepalive=60)
            except Exception as e:
                print(e)
            self._client.loop_forever(retry_first_connection=True)

    def RunAsync(self):
        self._client = mqtt.Client()
        self._client.on_connect = self._onConnect
        self._client.on_disconnect = self._onDisconnect
        self._client.on_message = self._onMessage
        self._client.username_pw_set(username=self._username, password=self._password)
        try:
            self._client.connect(host=self._host, port=self._port, keepalive=60)
        except Exception as e:
            print(e)
        self._client.loop_start()

class PingService:
    def __init__(self, deviceType, deviceId, brokerHost, brokerPort, brokerUserName, brokerPassword, interval=20, addIp=True) -> None:
        self._deviceType = deviceType
        self._deviceId = deviceId
        self._brokerHost = brokerHost
        self._brokerPort = brokerPort
        self._brokerUserName = brokerUserName
        self._brokerPassword = brokerPassword
        self._interval = interval
        self._addIp = addIp
        self._disconnectedEvent = threading.Event()
        self._connectedEvent = threading.Event()
        self._topic = "d2cping/{0}/{1}".format(self._deviceType, self._deviceId)
        self._channel = MqttChannel(host=brokerHost, 
                                    port=brokerPort, 
                                    username=brokerUserName, 
                                    password=brokerPassword)
        self._channel.on_connect = self._on_connected
        self._channel.on_disconnect = self._on_disconnected
        self._disconnectedEvent.set()
        self._isRunCalled = False
        self._client = None

    def _on_connected(self, client, userdata, falgs, rc):
        self._client = client
        self._connectedEvent.set()
        self._disconnectedEvent.clear()

    def _on_disconnected(self, client, userdata, rc):
        self._client = None
        self._connectedEvent.clear()
        self._disconnectedEvent.set()

    def Run(self):
        if self._isRunCalled is True:
            return
        self._isRunCalled = True
        self._channel.RunAsync()
        while True:
            pingMsg = {
                'interval': self._interval
            }
            if self._disconnectedEvent.wait(self._interval) is False:
                if self._addIp is True:
                    ipType, ipAddr = getIpV4()
                    if ipType is not None:
                        pingMsg['ip'] = ipAddr
                self._client.publish(topic=self._topic, payload=json.dumps(pingMsg),qos=0)
            else:
                time.sleep(1)


if __name__ == "__main__":
    p = PingService(
        deviceType="MockType",
        deviceId="01",
        brokerHost="localhost",
        brokerPort=1883,
        brokerUserName="mqtt_host:mqtt_test",
        brokerPassword="123456",
        addIp=False
    )
    p.Run()


from fileinput import filename
import logging
import logging.handlers
from pathlib import Path
import enum

try:
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass

class LogLevelEnum(enum.Enum):
    critical = logging.CRITICAL
    fatal = logging.FATAL
    error = logging.ERROR
    warning = logging.WARNING
    info = logging.INFO
    debug = logging.DEBUG
    notset = logging.NOTSET

class _logLevel:
    g_logLevel = None

def setLoggerLevel(level=LogLevelEnum.warning):
    _logLevel.g_logLevel = level

def getLogger(logName='default_vmilabs', savedPath=".",enableFile=False, enableConsole=False, logLevel=None):
    if logName == "":
        logName = 'default_vmilabs'
    if logLevel is None:
        logLevel = _logLevel.g_logLevel if _logLevel.g_logLevel is not None else LogLevelEnum.warning
    logLevel = logLevel.value
    p = Path(savedPath)
    if p.is_file():
        p = p.parent
    elif p.is_dir():
        pass
    elif p.exists() is False:
        p.mkdir(parents=True, exist_ok=True)
    _logger = logging.getLogger(logName)
    _logger.setLevel(logLevel)
    _logFileName = p / (logName + ".log")
    if enableFile is True:
        handler = logging.handlers.RotatingFileHandler(
            _logFileName,
            maxBytes= 50 * 1024 * 1023,
            backupCount=10
        )
        FORMAT = "[%(filename)s:%(lineno)s - %(funcName)s]: %(message)s"
        formatter = logging.Formatter(FORMAT)
        handler.setFormatter(formatter)
        _logger.addHandler(handler)
    
    if enableConsole is True:
        consoleHandler = logging.StreamHandler()
        FORMAT = "[%(filename)s:%(lineno)s - %(funcName)s]: %(message)s"
        formatter = logging.Formatter(FORMAT)
        consoleHandler.setFormatter(formatter)
        _logger.addHandler(consoleHandler)
    return _logger

class vmilog:
    def __init__(self, loggerName):
        self._loggerName = loggerName
        self._logger = logging.getLogger(loggerName)
        self._logger.addHandler(NullHandler())

    def enableFileHandler(self, handler=None, fileName=None):
        _logFileName = fileName
        if _logFileName is None:
            _logFileName = self._loggerName + "_d.log"
        if handler is None:
            handler = logging.handlers.RotatingFileHandler(
                _logFileName,
                maxBytes=50 * 1024 * 1024,
                backupCount= 10,
            )
            FORMAT = "[%(filename)s:%(lineno)s - %(funcName)s]: %(message)s"
            formatter = logging.Formatter(FORMAT)
            handler.setFormatter(formatter)
        self._logger.addHandler(handler)

    def enableConsoleHandler(self, handler=None):
        if handler is None:
            handler = logging.StreamHandler()
            FORMAT = "[%(filename)s:%(lineno)s - %(funcName)s]: %(message)s"
            formatter = logging.Formatter(FORMAT)
            handler.setFormatter(formatter)
        self._logger.addHandler(handler)
    
    def error(self,msg):
        self._logger.error(msg)
    
    def warning(self, msg):
        self._logger.warning(msg)
    
    def debug(self, msg):
        self._logger.debug(msg)
    
    def info(self, msg):
        self._logger.info(msg)
    
    def setLogLevel(self, level):
        obj = {
            "critical": logging.CRITICAL,
            "error": logging.ERROR,
            "warning": logging.WARNING,
            "info": logging.INFO,
            "debug": logging.DEBUG,
            "notset": logging.NOTSET
        }
        lowcaseLevel = level.lower()
        logLevel = obj.get(lowcaseLevel, logging.WARNING)
        self._logger.setLevel(logLevel)

_logger = vmilog(loggerName="default_vmilog")

def enableFileHandler(handler=None, fileName=None):
    _logger.enableFileHandler(handler=handler, fileName=fileName)

def enableConsoleHandler(handler=None):
    _logger.enableConsoleHandler(handler=handler)

def error(msg):
    _logger.error(msg)

def warning(msg):
    _logger.warning(msg)

def debug(msg):
    _logger.debug(msg)

def info(msg):
    _logger.info(msg)

def setLogLevel(level):
    _logger.setLogLevel(level=level)






from encodings import utf_8
import json
from pathlib import Path
import os, sys

_default = None

def getDefault():
    global _default
    return _default

def setDefault(defaultConfig):
    global _default
    _default = defaultConfig

def createDefaultFile(configFilePath=None):
    configFile = configFilePath
    if configFile is None:
        home = Path.home()
        vmiDir = home / '.vmi'
        configFile = vmiDir / 'config.json'
        if vmiDir.is_dir() is False:
            vmiDir.mkdir(parents=True, exist_ok=True)
    else:
        configFile = Path(configFilePath)
    defaultConfig = getDefault()
    configFile.write_text(json.dumps(defaultConfig, ensure_ascii=False, indent=4), encoding="utf8")
    return defaultConfig

def getConfiguration(configFilePath=None, shouldCreateDefault= False):
    configFile = configFilePath
    vmiDir = None
    if configFile is None:
        home = Path.home()
        vmiDir = home / '.vmi'
        configFile = vmiDir / 'config.json'
        if vmiDir.is_dir() is False:
            vmiDir.mkdir(parents=True, exist_ok=True)
    else:
        configFile = Path(configFilePath)
        vmiDir = configFile.parents[0]

    if configFile.is_file() is False:
        if shouldCreateDefault is True:
            return createDefaultFile(configFilePath=configFilePath), vmiDir
        else:
            return None, None
    
    configuration = configFile.read_text(encoding='utf-8')
    try:
        configDict = json.loads(configuration)
    except Exception:
        configDict = createDefaultFile()
    finally:
        return configDict, vmiDir